@@ -1,5 +1,10 @@
-#ifndef LIBCRYPTOCOMPAT_ARC4RANDOM_H
-#define LIBCRYPTOCOMPAT_ARC4RANDOM_H
+/*
+ * Public domain
+ * arc4random.h compatibility shim
+ */
+
+#ifndef IKED_COMPAT_ARC4RANDOM_H
+#define IKED_COMPAT_ARC4RANDOM_H
 
 #if defined(_AIX)
 #include "arc4random_aix.h"
