@@ -1,4 +1,4 @@
-/*	$OpenBSD: ikectl.c,v 1.30 2022/12/04 11:54:31 tobhe Exp $	*/
+/*	$OpenBSD: ikectl.c,v 1.34 2024/11/21 13:21:34 claudio Exp $	*/
 
 /*
  * Copyright (c) 2007-2013 Reyk Floeter <reyk@openbsd.org>
@@ -45,7 +45,6 @@ struct imsgname {
 };
 
 struct imsgname *monitor_lookup(uint8_t);
-void		 monitor_id(struct imsg *);
 int		 monitor(struct imsg *);
 
 int		 show_string(struct imsg *);
@@ -251,7 +250,7 @@ main(int argc, char *argv[])
 	else
 		if ((ibuf = malloc(sizeof(struct imsgbuf))) == NULL)
 			err(1, "malloc");
-	imsg_init(ibuf, ctl_sock);
+	imsgbuf_init(ibuf, ctl_sock);
 
 	/* process user request */
 	switch (res->action) {
@@ -340,13 +339,12 @@ main(int argc, char *argv[])
 		break;
 	}
 
-	while (ibuf->w.queued)
-		if (msgbuf_write(&ibuf->w) <= 0 && errno != EAGAIN)
-			err(1, "write error");
+	if (imsgbuf_flush(ibuf) == -1)
+		err(1, "write error");
 
 	while (!done) {
-		if ((n = imsg_read(ibuf)) == -1 && errno != EAGAIN)
-			errx(1, "imsg_read error");
+		if ((n = imsgbuf_read(ibuf)) == -1)
+			errx(1, "imsgbuf_read error");
 		if (n == 0)
 			errx(1, "pipe closed");
 
