@@ -1,4 +1,4 @@
-/*	$OpenBSD: ikev2_msg.c,v 1.100 2023/08/04 19:06:25 claudio Exp $	*/
+/*	$OpenBSD: ikev2_msg.c,v 1.103 2024/11/21 13:26:49 claudio Exp $	*/
 
 /*
  * Copyright (c) 2019 Tobias Heider <tobias.heider@stusta.de>
@@ -300,7 +300,7 @@ ikev2_msg_send(struct iked *env, struct iked_message *msg)
 			log_debug("%s: failed to set NAT-T", __func__);
 			return (-1);
 		}
-		if (ibuf_add_buf(new, buf) == -1) {
+		if (ibuf_add_ibuf(new, buf) == -1) {
 			ibuf_free(new);
 			log_debug("%s: failed to set NAT-T", __func__);
 			return (-1);
@@ -781,7 +781,7 @@ ikev2_msg_send_encrypt(struct iked *env, struct iked_sa *sa, struct ibuf **ep,
 		log_debug("%s: encryption failed", __func__);
 		goto done;
 	}
-	if (ibuf_add_buf(buf, e) != 0)
+	if (ibuf_add_ibuf(buf, e) != 0)
 		goto done;
 
 	/* Add integrity checksum (HMAC) */
@@ -889,7 +889,7 @@ ikev2_send_encrypted_fragments(struct iked *env, struct iked_sa *sa,
 			log_debug("%s: encryption failed", __func__);
 			goto done;
 		}
-		if (ibuf_add_buf(buf, e) != 0)
+		if (ibuf_add_ibuf(buf, e) != 0)
 			goto done;
 
 		/* Add integrity checksum (HMAC) */
@@ -963,7 +963,7 @@ ikev2_msg_auth(struct iked *env, struct iked_sa *sa, int response)
 
 	if ((authmsg = ibuf_dup(buf)) == NULL)
 		return (NULL);
-	if (ibuf_add_buf(authmsg, nonce) != 0)
+	if (ibuf_add_ibuf(authmsg, nonce) != 0)
 		goto fail;
 
 	if ((hash_setkey(sa->sa_prf, ibuf_data(prfkey),
@@ -1275,15 +1275,25 @@ ikev2_msg_lookup(struct iked *env, struct iked_msgqueue *queue,
 
 int
 ikev2_msg_retransmit_response(struct iked *env, struct iked_sa *sa,
-    struct iked_message *msg, uint8_t exchange)
+    struct iked_message *msg, struct ike_header *hdr)
 {
 	struct iked_msg_retransmit	*mr = NULL;
-	struct iked_message	*m = NULL;
+	struct iked_message		*m = NULL;
 
-	if ((mr = ikev2_msg_lookup(env, &sa->sa_responses, msg, exchange))
-	    == NULL)
+	if ((mr = ikev2_msg_lookup(env, &sa->sa_responses, msg,
+	    hdr->ike_exchange)) == NULL)
 		return (-2);	/* not found */
 
+	if (hdr->ike_nextpayload == IKEV2_PAYLOAD_SKF) {
+		/* only retransmit for fragment number one */
+		if (ikev2_pld_parse_quick(env, hdr, msg,
+		    msg->msg_offset) != 0 || msg->msg_frag_num != 1) {
+			log_debug("%s: ignoring fragment", SPI_SA(sa, __func__));
+			return (0);
+		}
+		log_debug("%s: first fragment", SPI_SA(sa, __func__));
+	}
+
 	TAILQ_FOREACH(m, &mr->mrt_frags, msg_entry) {
 		if (sendtofrom(m->msg_fd, ibuf_data(m->msg_data),
 		    ibuf_size(m->msg_data), 0,
@@ -1295,7 +1305,7 @@ ikev2_msg_retransmit_response(struct iked *env, struct iked_sa *sa,
 		}
 		log_info("%sretransmit %s res %u local %s peer %s",
 		    SPI_SA(sa, NULL),
-		    print_map(exchange, ikev2_exchange_map),
+		    print_map(hdr->ike_exchange, ikev2_exchange_map),
 		    m->msg_msgid,
 		    print_addr(&m->msg_local),
 		    print_addr(&m->msg_peer));
