@@ -1,4 +1,4 @@
-/*	$OpenBSD: util.c,v 1.43 2023/07/28 11:23:03 claudio Exp $	*/
+/*	$OpenBSD: util.c,v 1.45 2024/07/01 14:15:15 yasuoka Exp $	*/
 
 /*
  * Copyright (c) 2021 Tobias Heider <tobhe@openbsd.org>
@@ -735,16 +735,16 @@ prefixlen2mask6(uint8_t prefixlen, uint32_t *mask)
 const char *
 print_addr(void *addr)
 {
-	static char	 sbuf[IKED_CYCLE_BUFFERS][NI_MAXHOST + 7];
+	static char	 sbuf[IKED_CYCLE_BUFFERS][NI_MAXHOST + 9];
 	static int	 idx;
 	struct sockaddr	*sa = addr;
-	char		*buf;
-	size_t		 len;
+	char		*buf, *hbuf;
+	size_t		 len, hlen;
 	char		 pbuf[7];
 	in_port_t	 port;
 
-	buf = sbuf[idx];
-	len = sizeof(sbuf[idx]);
+	hbuf = buf = sbuf[idx];
+	hlen = len = sizeof(sbuf[idx]);
 	if (++idx >= IKED_CYCLE_BUFFERS)
 		idx = 0;
 
@@ -753,13 +753,21 @@ print_addr(void *addr)
 		return (buf);
 	}
 
+	if ((port = socket_getport(sa)) != 0 && sa->sa_family == AF_INET6) {
+		/* surround [] */
+		*(hbuf++) = '[';
+		hlen--;
+	}
+
 	if (getnameinfo(sa, SA_LEN(sa),
-	    buf, len, NULL, 0, NI_NUMERICHOST) != 0) {
+	    hbuf, hlen, NULL, 0, NI_NUMERICHOST) != 0) {
 		strlcpy(buf, "unknown", len);
 		return (buf);
 	}
 
-	if ((port = socket_getport(sa)) != 0) {
+	if (port != 0) {
+		if (sa->sa_family == AF_INET6)
+			(void)strlcat(buf, "]", len);
 		snprintf(pbuf, sizeof(pbuf), ":%d", port);
 		(void)strlcat(buf, pbuf, len);
 	}
@@ -792,8 +800,7 @@ print_proto(uint8_t proto)
 	if ((p = getprotobynumber(proto)) != NULL)
 		strlcpy(buf[idx], p->p_name, sizeof(buf[idx]));
 	else
-		snprintf(buf[idx], sizeof(buf), "%u", proto);
-
+		snprintf(buf[idx], sizeof(buf[idx]), "%u", proto);
 
 	return (buf[idx++]);
 }
